#include "Search.h"
#include "Bits.h"

Search::Search(char* fen) :
		GenMoves(fen) {
	eval = new Eval(NULL, this);
	null_sem = false;
}
void Search::startClock() {
	ftime(&start_time);
}
void Search::setRandomParam() {
	eval->setRandomParam();
}
void Search::setMainPly(int m) {
	mainDepth = m;
}

void Search::writeParam(char* param_file, int cd_param, bool append) {
	eval->writeParam(param_file, cd_param, append);
}
int Search::checkTime() {
	if (running == 2)
		return 2;
	struct timeb t_current;
	ftime(&t_current);
	return ((int) (1000 * (t_current.time - start_time.time))) >= maxTimeMillsec ? 0 : 1;
}

int Search::getScore(int side) {
	return eval->score(side);
}
Search::~Search() {
	delete eval;
}
int Search::quiescence(int alpha, const int side, int beta, const char promotion_piece, int dep
#if defined(PERFT_MODE) || defined(HASH_MODE)
		, u64 key
#endif
		) {
#if defined(PERFT_MODE) || defined(HASH_MODE)
	u64 key2 = key;
#endif
	if (!running)
		return 0;
#ifdef DEBUG_MODE
	ASSERT(chessboard[KING_BLACK]);
	ASSERT(chessboard[KING_WHITE]);
	if ((!side && !chessboard[KING_BLACK]) || (side && !chessboard[KING_WHITE])) {
		ASSERT(0);
	}
#endif
	int score = -_INFINITE;
	if (!(num_movesq++ & 1023))
		running = checkTime();
	int xside, listcount;
	Tmove * mossa;
	score = eval->score(side
#ifdef FP_MODE
			, alpha, beta
#endif
			);
	if (score >= beta) {
		return beta;
	}
#ifdef FP_MODE
	/**************Delta Pruning ****************/
	char fprune = 0;
	int fscore;
	if ((fscore = score + (promotion_piece == -1 ? VALUEQUEEN : 2 * VALUEQUEEN))
			< alpha) {
		fprune = 1;
	}
	/************ end Delta Pruning *************/
#endif
	xside = side ^ 1;
	if (score > alpha)
		alpha = score;
	list_id++;
	ASSERT(list_id < MAX_PLY);
	if (generateCap(STANDARD_MOVE_MASK, side)) {
		gen_list[list_id--][0].score = 0;
		return _INFINITE - (mainDepth - dep);
	}
	listcount = gen_list[list_id][0].score;
	if (!listcount) {
		--list_id;
		return score;
	}
	int i;
	while ((i = getNextMove(gen_list[list_id])) != -1) {
		mossa = &gen_list[list_id][i];
		makemove(mossa
#if defined(PERFT_MODE) || defined(HASH_MODE)
				, &key
#endif
				);
#ifdef FP_MODE
		/**************Delta Pruning ****************/
		if (fprune && ((mossa->type & 0x3) != PROMOTION_MOVE_MASK) /*&& mossa->capture == 12 */
				&& fscore + PIECES_VALUE[mossa->capture] <= alpha /*&& !in_check (side)*/) {
			INC(n_cut_fp);
			takeback(mossa

			);
#if defined(PERFT_MODE) || defined(HASH_MODE)
			key = key2;
#endif
			continue;
		}
		/************ end Delta Pruning *************/
#endif
		int val = -quiescence(-beta, xside, -alpha, mossa->promotion_piece, dep + 1
#if defined(PERFT_MODE) || defined(HASH_MODE)
				, key
#endif
				);
		score = max(score, val);
		takeback(mossa

		);
#if defined(PERFT_MODE) || defined(HASH_MODE)
		key = key2;
#endif
		if (score >= beta) {
			gen_list[list_id--][0].score = 0;
			return beta;
		}
		if (score > alpha) {
			alpha = score;
		}
	}
	gen_list[list_id--][0].score = 0;
	return alpha;
}
void Search::setRunning(int r) {
	running = r;
}
int Search::getRunning() {
	return running;
}

void Search::setMaxTimeMillsec(int n) {
	maxTimeMillsec = n;
}

int Search::getMaxTimeMillsec() {
	return maxTimeMillsec;
}
int Search::search(const int side, int depth, int alpha, int beta, LINE * pline
#ifdef DEBUG_MODE
		, Tmove *Pmossa
#endif
#if defined(PERFT_MODE) || defined(HASH_MODE)
		, const u64 zobristKey
#endif
		) {
	if (!running)
		return 0;
	running = checkTime();
	char hashf = hashfALPHA;
	int score = -_INFINITE;
	LINE line;
	line.cmove = 0;
	//transposition table allocated in GenMoves.cpp:8, clean in IterativeDeeping.cpp:100

	Thash * phashe = &hash_array[zobristKey % HASH_SIZE];
#ifdef TT_MODE
	if (phashe->key == zobristKey && phashe->depth >= depth) {
		switch (phashe->flags) {
			case hashfEXACT:
			updatePv(pline, &line, &phashe->best);
			return phashe->score;
			break;
			case hashfBETA:
			if (phashe->score >= beta) {
				return phashe->score;
			}
			//alpha = max(alpha, phashe->score);
			break;
			case hashfALPHA:
			if (phashe->score < alpha) {
				return phashe->score;
			}
			//beta = min(beta, phashe->score);
			break;
		}
	}
#endif

	int n_pieces_x_side = n_pieces(side ^ 1);
	int n_pieces_side = n_pieces(side);
	int is_incheck_side = in_check(side);

	if (depth == 0) {
		score = quiescence(alpha, side, beta, -1, 0, zobristKey);
		pline->cmove = 0;
		return score;
	}
	Tmove* move;
	Tmove* best;

	//////////////////////////

	Friend_king[side] = BITScanForward(chessboard[KING_BLACK + side]);
	Friend_king[side ^ 1] = BITScanForward(chessboard[KING_BLACK + (side ^ 1)]);
	list_id++;
	//generate capture
	if (generateCap(STANDARD_MOVE_MASK, side)) { //add to gen_list
		gen_list[list_id--][0].score = 0; //reset list
		//mate!
		return _INFINITE - (mainDepth - depth + 1);
	}
	generateMoves(STANDARD_MOVE_MASK, side); //add to gen_list
	if (!gen_list[list_id][0].score) {
		--list_id;
		if (is_incheck_side) //no moves available
			return -_INFINITE + (mainDepth - depth + 1);
		return 0;
	}

	int ii;
	bool firstMove = false;
	while ((ii = getNextMove(gen_list[list_id])) != -1) { //get all moves
		move = &gen_list[list_id][ii];
		if (!firstMove) {
			best = move;
			firstMove = true;
		}
		makemove(move);
		int val = -search(side ^ 1, depth - 1, -beta, -alpha, &line, makeZobristKey(side ^ 1));

		if (val > score) {
			best = move;
			score = val;
		}
		takeback(move);

		if (score >= beta) {
			RecordHash(depth, hashfBETA, zobristKey, score, move);
			gen_list[list_id--][0].score = 0; //reset list
			return score;
		}
		if (score > alpha) {
			alpha = score;
			hashf = hashfEXACT;
			best = move;
			updatePv(pline, &line, move);
		}
	}
	RecordHash(depth, hashf, zobristKey, score, best);
	gen_list[list_id--][0].score = 0; //reset list
	return score;
}

void Search::updatePv(LINE * pline, const LINE * line, const Tmove * mossa) {
#ifdef DEBUG_MODE
	ASSERT(line->cmove < MAX_PLY - 1);
#endif
	memcpy(&(pline->argmove[0]), mossa, sizeof(Tmove));
	memcpy(pline->argmove + 1, line->argmove, line->cmove * sizeof(Tmove));
#ifdef DEBUG_MODE
	assert(line->cmove >= 0);
#endif
	pline->cmove = line->cmove + 1;
}
void Search::RecordHash(const char depth, const char flags, const u64 key, const int score, Tmove* bestMove) {
	ASSERT(key);
	if (bestMove->from < 0 || bestMove->from > 63 || bestMove->to < 0 || bestMove->to > 63)
		return;
	if ((bestMove->from == 0 && bestMove->to == 0) )
		return;

	Thash * phashe = &hash_array[key % HASH_SIZE];
	if (phashe->key && phashe->depth >= depth)
		return;

	phashe->key = key;
	phashe->score = score;
	phashe->flags = flags;
	phashe->depth = depth;

	memcpy(&(phashe->best), bestMove, sizeof(Tmove));

}

