/*
Copyright (C) 2008-2012  giuseppe.cannella@gmail.com
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include "maindefine.h"
#include "Perft.h"
#include "Uci.h"
#include "GenMoves.h"
#include "Test.h"
#include <unistd.h>
void dispose() {
	free(Bits::ROTATE_LEFT);
	free(Bits::ROTATE_RIGHT);
	free(Bits::BITCOUNT);
}
void start() {
#ifdef DEBUG_MODE
	cout << "DEBUG_MODE\n";
#endif
#ifndef HASH_MODE
	cout << "NO HASH_MODE\n";
#endif
#ifndef NULL_MODE
//	cout << "NO NULL_MODE\n";
#endif
#ifndef FP_MODE
//	cout << "NO FP_MODE\n";
#endif
	Bits::initBitCount();
	srand(time(NULL));
}
#ifdef CRAFTY_TEST_MODE
int main(int argc, char **argv) {
	start();
	Test t;

	t.test_eval_crafty_tuning("/home/geko/chess/book1.epd", false);	return 0;
	//t.test_eval_crafty("/home/geko/chess/openings.epd");	return 0;

}
#else

#ifdef PERFT_MODE
int main(int argc, char **argv) {// perft
	start();
	char opt;
	int nCpu = 0;
	int perftDepth = 0;
	char *fen = INITIAL_FEN;
	int PERFT_HASH_SIZE = 0;//16777213;
	while ((opt = getopt(argc, argv, "d:p:h:f:")) != -1) {
		switch (opt) {
			case 'd': //depth
			perftDepth = atoi(optarg);
			break;
			case 'p'://N cpu
			nCpu = atoi(optarg);
			break;
			case 'h'://hash
			PERFT_HASH_SIZE = atoi(optarg);
			break;
			case 'f'://fen
			fen = optarg;
			break;
		}
	}

	if (perftDepth > MAX_PLY || perftDepth <= 0)
	perftDepth = 5;
	if (nCpu > 8 || nCpu <= 0)
	nCpu = 1;
	cout << "-d depth\n-p nCpu\n-h hash size (Mb)\n-f FEN" << endl;
	cout << "butterfly -d " << perftDepth << " -p " << nCpu << " -h " << PERFT_HASH_SIZE << " -f \"" << fen << "\"" << endl;

	RootPerft* p = new RootPerft(fen, perftDepth, nCpu, PERFT_HASH_SIZE);
	delete p;

	dispose();
	return 0;
}
#else
int main(int argc, char **argv) {
	char opt;
	char* iniFile = NULL;
	while ((opt = getopt(argc, argv, "w:r:")) != -1) {
		switch (opt) {
		case 'r':
			iniFile = optarg;
			break;
		case 'w':
			Eval eval(NULL, NULL);
			eval.writeParam(optarg, 1, false);
			return 0;
		}
	}
	start();
	Uci* uci = new Uci(iniFile);
	delete uci;
	dispose();
	return 0;
}
#endif
#endif

